<?php
/* Copyright start
  Copyright (C) 2008 - 2025 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace App\Entity\ViewTemplate;

use App\Traits\Trackable;
use Doctrine\ORM\Mapping as ORM;
use App\Repository\DashboardRepository;
use ApiPlatform\Core\Annotation\ApiResource;
use Symfony\Component\Serializer\Annotation\Groups;

/**
 *  @ApiResource(
 *     attributes={"validation_groups"={"fsr_primary", "fsr_all"}, "fsr_type"="dashboard", "fsr_module"="dashboard", "fsr_singular"="dashboard"},
 *     denormalizationContext={"groups"={"fsr_all", "password"}, "fetch_data"=false, "fsr_type"="dashboard"},
 *     collectionOperations={
 *          "get" = { "path"="dashboard", "security_post_denormalize" = "is_granted('read.dashboard', object)" },
 *          "post" = { "path"="dashboard", "security_post_denormalize" = "is_granted('create.dashboard', object)" }
 *     },
 *     itemOperations={
 *          "get" = { "path"="dashboard/{uuid}", "security" = "is_granted('read.dashboard', object)" },
 *          "put" = { "path"="dashboard/{uuid}", "security" = "is_granted('update.dashboard', object)" },
 *          "delete" = { "path"="dashboard/{uuid}", "security" = "is_granted('delete.dashboard', object)" }
 *     }
 * )
 * @ORM\Entity(repositoryClass=DashboardRepository::class)
 * @ORM\Table(name="generic_template")
 */
class Dashboard extends GenericTemplate
{
    use Trackable;

    /**
     * @Groups({"fsr_primary", "fsr_all", "isDefault"})
     * @ORM\Column(type="boolean", nullable=true, options={"default" = false})
     */
    private $isDefault = false;

    /**
     * Get the value of isDefault
     */
    public function getIsDefault()
    {
        return $this->isDefault;
    }

    /**
     * Set the value of isDefault
     */
    public function setIsDefault($isDefault): self
    {
        $this->isDefault = $isDefault;

        return $this;
    }
}
