<?php
/* Copyright start
  Copyright (C) 2008 - 2025 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace App\Entity\ViewTemplate;

use App\Traits\Importable;
use App\Entity\Base\BaseEntity;
use Doctrine\ORM\Mapping as ORM;
use Doctrine\ORM\Mapping\UniqueConstraint;
use ApiPlatform\Core\Annotation\ApiResource;
use App\Repository\SystemViewTemplateRepository;
use Symfony\Component\Serializer\Annotation\Groups;

/**
 *  @ApiResource(
 *     attributes={"validation_groups"={"fsr_primary", "fsr_all"}, "fsr_type"="system_view_templates", "fsr_module"="application", "fsr_singular"="system_view_template"},
 *     denormalizationContext={"groups"={"fsr_all", "password"}, "fetch_data"=false, "fsr_type"="system_view_templates"},
 *     collectionOperations={
 *          "get" = { "security_post_denormalize" = "is_granted('read.application', object)" },
 *          "post" = { "security_post_denormalize" = "is_granted('create.application', object)" }
 *     },
 *     itemOperations={
 *          "get" = { "security" = "is_granted('read.application', object)" },
 *          "put" = { "security" = "is_granted('update.application', object)" },
 *          "delete" = { "security" = "is_granted('delete.application', object)" }
 *     }
 * )
 * @ORM\Entity(repositoryClass=SystemViewTemplateRepository::class)
 * @ORM\Table(name="cyops.system_view_template", schema="cyops", uniqueConstraints={@UniqueConstraint(name="system_module_view_name_unique", columns={"module", "viewOptions", "name"})})

 */
class SystemViewTemplate extends BaseEntity
{
    use Importable;

    /**
     * @Groups({"fsr_primary", "fsr_all", "type"})
     * @ORM\Column(type="string", length=255, nullable=true)
     */
    private $type;

    /**
     * @Groups({"fsr_primary", "fsr_all", "config"})
     * @ORM\Column(type="json", nullable=true)
     */
    private $config = [];

    /**
     * @Groups({"fsr_primary", "fsr_all", "module"})
     * @ORM\Column(type="string", length=255, nullable=false)
     */
    private $module;

    /**
     * @Groups({"fsr_primary", "fsr_all", "filters"})
     * @ORM\Column(type="jsonb", nullable=true)
     */
    private $filters = [];

    /**
     * @Groups({"fsr_primary", "fsr_all", "isDefault"})
     * @ORM\Column(type="boolean", nullable=true, options={"default" = false})
     */
    private $isDefault = false;

    /**
     * @Groups({"fsr_primary", "fsr_all", "visible"})
     * @ORM\Column(type="boolean", nullable=true, options={"default" = true})
     */
    private $visible = true;

    /**
     * @Groups({"fsr_primary", "fsr_all", "viewOptions"})
     * @ORM\Column(type="string", length=255, nullable=false)
     */
    private $viewOptions;

    /**
     * @Groups({"fsr_primary", "fsr_all", "name"})
     * @ORM\Column(type="string", length=255, nullable=false)
     */
    private $name;

    /**
     * @Groups({"fsr_primary", "fsr_all", "system"})
     * @ORM\Column(type="boolean", nullable=true, options={"default" = false})
     */
    private $system = false;

    public function getType(): ?string
    {
        return $this->type;
    }

    public function setType(?string $type): self
    {
        $this->type = $type;

        return $this;
    }

    public function getConfig(): ?array
    {
        return $this->config;
    }

    public function setConfig(?array $config): self
    {
        $this->config = $config;

        return $this;
    }

    public function getModule(): ?string
    {
        return $this->module;
    }

    public function setModule(?string $module): self
    {
        $this->module = $module;

        return $this;
    }

    public function getFilters(): ?array
    {
        return $this->filters;
    }

    public function setFilters(?array $filters): self
    {
        $this->filters = $filters;

        return $this;
    }

    public function getIsDefault(): ?bool
    {
        return $this->isDefault;
    }

    public function setIsDefault(?bool $isDefault): self
    {
        $this->isDefault = $isDefault;

        return $this;
    }

    public function getViewOptions(): ?string
    {
        return $this->viewOptions;
    }

    public function setViewOptions(?string $viewOptions): self
    {
        $this->viewOptions = $viewOptions;

        return $this;
    }

    public function getName(): ?string
    {
        return $this->name;
    }

    public function setName(?string $name): self
    {
        $this->name = $name;

        return $this;
    }

    public function getSystem(): ?bool
    {
        return $this->system;
    }

    public function setSystem(?bool $system): self
    {
        $this->system = $system;

        return $this;
    }

    /**
     * Get the value of visible
     */
    public function getVisible()
    {
        return $this->visible;
    }

    /**
     * Set the value of visible
     */
    public function setVisible(?bool $visible): self
    {
        $this->visible = $visible;

        return $this;
    }
}
