<?php
/* Copyright start
  Copyright (C) 2008 - 2024 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

declare(strict_types=1);

namespace App\Migrations;

use App\Entity\ViewTemplate\SystemViewTemplate;
use App\Entity\ViewTemplate\UserViewTemplate;
use Ramsey\Uuid\Uuid;
use Doctrine\DBAL\Schema\Schema;
use Doctrine\Migrations\AbstractMigration;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\DependencyInjection\ContainerAwareInterface;

/**
 * Auto-generated Migration: Please modify to your needs!
 */
final class Version20241217061237 extends AbstractMigration implements ContainerAwareInterface
{

    /** @var ContainerInterface */
    private $container;

    public function setContainer(?ContainerInterface $container = null)
    {
        $this->container = $container;
    }

    public function getDescription(): string
    {
        return 'Migration of old system view templates';
    }

    public function up(Schema $schema): void
    {
        $system_view_template = "cyops.system_view_template";
        $user_view_template = "cyops.user_view_template";

        $allSystemViewTemplates = $this->connection->fetchAllAssociative("SELECT * FROM $system_view_template;");
        if (count($allSystemViewTemplates)) {
            $backupsToRemove = [];

            $managerRegistry = $this->container->get('doctrine');
            $entityManager = $managerRegistry->getManagerForClass(SystemViewTemplate::class);

            foreach ($allSystemViewTemplates as $systemViewTemplate) {
                $oldUuid = $systemViewTemplate['uuid'];
                if (!Uuid::isValid($oldUuid)) {
                    if ($this->checkIfBackup($oldUuid)) {
                        $backupsToRemove[] = $oldUuid;
                        continue;
                    }
                    $migratedSvtData = $this->migrateToNewVtStructure($systemViewTemplate);
                    if ($migratedSvtData) {
                        $newUuid = $migratedSvtData['newUuid'];
                        $oldUuid = $migratedSvtData['oldUuid'];
                        $name = $migratedSvtData['name'];
                        $module = $migratedSvtData['module'];
                        $viewOptions = $migratedSvtData['viewOptions'];
                        $filters = $migratedSvtData['filters'];
                        $isDefault = $migratedSvtData['isDefault'];
                        $importedBy = $migratedSvtData['importedBy'];

                        $svtName = $viewOptions === 'settings' ? "Old View Setting" : "Old View Template";
                        $default = 'false';
                        $updateQuery = "UPDATE $system_view_template SET name='$svtName',module='$module',filters='$filters',viewOptions='$viewOptions',isDefault=$default,importedBy='$importedBy', visible=false WHERE uuid='$oldUuid';";
                        $this->addSql($updateQuery);
                        $type = array_key_exists('type', $systemViewTemplate) ? $systemViewTemplate['type'] : '';

                        $svt = new SystemViewTemplate();
                        $svt->setUuid($newUuid);
                        $svt->setName($name);
                        $svt->setModule($module);
                        $svt->setViewOptions($viewOptions);
                        $svt->setIsDefault($isDefault);
                        $svt->setType($type);
                        $svt->setConfig(json_decode($systemViewTemplate['config'], true) ?? []);
                        $svt->setVisible(true);
                        $entityManager->persist($svt);
                    }
                }
            }
            $entityManager->flush();

            if (!empty($backupsToRemove)) {
                foreach ($backupsToRemove as $backupSvtUuid) {
                    $query = "DELETE FROM $system_view_template WHERE uuid='$backupSvtUuid';";
                    $this->addSql($query);
                }
            }
        }
        $entityManager = $managerRegistry->getManagerForClass(UserViewTemplate::class);
        $allUserViewTemplates = $this->connection->fetchAllAssociative("SELECT * FROM $user_view_template;");
        if (count($allUserViewTemplates)) {
            foreach ($allUserViewTemplates as $userViewTemplate) {
                $migratedSvtData = $this->migrateToNewVtStructure($userViewTemplate);
                if ($migratedSvtData) {
                    $newUuid = $migratedSvtData['newUuid'];
                    $oldUuid = $migratedSvtData['oldUuid'];
                    $module = $migratedSvtData['module'];
                    $viewOptions = $migratedSvtData['viewOptions'];
                    $filters = $migratedSvtData['filters'];
                    $isDefault = $migratedSvtData['isDefault'];
                    $name = array_key_exists('name', $userViewTemplate) ? $userViewTemplate['name'] : "Preserved Template";

                    $svtName = "Old View Template";
                    $default = 'false';
                    $updateQuery = "UPDATE $user_view_template SET name='$svtName', module='$module',filters='$filters',viewOptions='$viewOptions',isDefault=$default WHERE uuid='$oldUuid';";
                    $this->addSql($updateQuery);

                    $type = array_key_exists('type', $userViewTemplate) ? $userViewTemplate['type'] : '';
                    $uvt = new UserViewTemplate();
                    $uvt->setUuid($newUuid);
                    $uvt->setName($name);
                    $uvt->setModule($module);
                    $uvt->setViewOptions($viewOptions);
                    $uvt->setIsDefault($isDefault);
                    $uvt->setOwner($userViewTemplate['owner']);
                    $uvt->setType($type);
                    $uvt->setConfig(json_decode($userViewTemplate['config'], true) ?? []);
                    $uvt->setVisible(true);
                    $entityManager->persist($uvt);
                }
            }
            $entityManager->flush();
        }
    }

    private function migrateToNewVtStructure($viewTemplate)
    {
        $migratedSvtData = [];
        $oldUuid = $viewTemplate['uuid'];
        // Handling of old SVT structure migration
        if (!Uuid::isValid($oldUuid)) {
            $newUuid = Uuid::uuid4()->toString();
            $name = "Preserved Template";
            $module = null;
            $viewOptions = null;
            $filters = json_encode([]);
            $isDefault = true;
            $importedBy = json_encode([]);

            if (str_starts_with($oldUuid, 'modules')) {
                $explodedOldUuid = explode('-', $oldUuid);
                $module = $explodedOldUuid[1];
                $viewOptions = $explodedOldUuid[2];
                $name = $viewOptions === 'settings' ? "Preserved Setting" : $name;
            } else if (str_starts_with($oldUuid, 'pages')) {
                $name = $oldUuid;
                $explodedOldUuid = explode('-', $oldUuid);
                $module = $explodedOldUuid[1];
                $viewOptions = 'detail';
            } else {
                $module = $oldUuid;
                $viewOptions = $module === 'grid_columns' ? 'list' : 'detail';
            }
            $migratedSvtData = [
                'newUuid' => $newUuid,
                'oldUuid' => $oldUuid,
                'name' => $name,
                'module' => $module,
                'viewOptions' => $viewOptions,
                'filters' => $filters,
                'isDefault' => $isDefault,
                'importedBy' => $importedBy
            ];
        }

        // Handling of old UVT structure migration
        if (array_key_exists('owner', $viewTemplate) && !isset($viewTemplate['module'])) {
            $newUuid = Uuid::uuid4()->toString();
            $oldUuid = $viewTemplate['uuid'];
            $module = $viewTemplate['name'];
            $viewOptions = $module === 'grid_columns' ? 'list' : 'detail';
            $filters = json_encode([]);
            $isDefault = true;
            $migratedSvtData = [
                'newUuid' => $newUuid,
                'oldUuid' => $oldUuid,
                'module' => $module,
                'viewOptions' => $viewOptions,
                'filters' => $filters,
                'isDefault' => $isDefault
            ];
        }

        return $migratedSvtData;
    }

    private function checkIfBackup($uuid)
    {
        return str_ends_with($uuid, '-backup') ? true : false;
    }

    public function down(Schema $schema): void
    {
        // this down() migration is auto-generated, please modify it to your needs

    }
}
